%
% Jacobiverfahren
% mit schlechtem Laufzeitverhalten
% ---------------------------------
%
%  Verfahren:
%    Iterationsmatrix: 
%                H = I-D^(-1)*A
%    Iterationsvorschrift: 
%          x_(k+1) = xk - D^(-1)*(b-A*xk)
%
% Inputvariablen
%  - A    Zu untersuchende Matrix
%  - b    Lsg von Ax=b (Spaltenvektor)
%  - x0   Startvektor  (Spaltenvektor)
%  - imax Anzahl maximaler Iterationsschritte
%  - tol  Genauigkeit, bei deren Erreichen die Iteration abbricht
%         (Durch vergleich der relativen Residuen-Norm)
%
% Outputvariablen
%  - xk         Die letzte Iterierte
%  - xit        Die Iterierten (inklusive Startvektor)
%               (spaltenweise)
%  - resnormvec Relative Residuennormen der Iterierten (inklusive Startvektor)
%               (Spaltenvektor)
%
% Programmaufruf mit:
%
%   [xk,xit,resnormvec] = jacobi_schlecht(A,b,x0,imax,tol)
%
%   Die Parameter x0,imax und tol sind optional, falls nicht angegeben
%   werden die folgenden Standardwerte genutzt:
%
%     x0   = Nullvektor
%     imax = 1000
%     tol  = 1e-15
%
%
function  [xk,xit,resnormvec] = jacobi_schlecht(A,b,x0,imax,tol)

%% Falls nicht angegeben, Standardwerte festlegen
if (nargin < 2)
  error('jacobi:NotEnoughInputs', 'Not enough input arguments.');
end
if (nargin < 5)
  tol = 1e-15;
  if (nargin < 4)
    imax = 1000;
    if (nargin < 3)
      x0 = zeros(size(A,1),1);
    end
  end
end
xk = x0;

%% Initialisierung
iter = 0;
res = b-A*xk;
resnorm = norm(res);
bnorm = norm(b);
n = size(A,1);

% Analyseinitialisierung
xit = xk;
resnormvec = resnorm/bnorm;

%% Iteration
while (iter < imax) && (resnorm/bnorm>tol)
    % neue Iterierte
    for i=1:n
        if mod(i,50)==0
            display(sprintf('iter=%d i=%d',iter,i));
        end
        summe=0;
        for j=1:n
            if i ~= j
                summe = summe + A(i,j)*xk(j);
            end
        end
        xk(i)=(b(i)-summe)/A(i,i);
    end
    display('Iterierte berechnet...');
    % neues Residuum
    res = b-A*xk;
    resnorm = norm(res);
    % Fuer spaetere Analyse
    xit = [xit,xk];
    resnormvec = [resnormvec;resnorm/bnorm];
    iter = iter+1;
end