% Steepest Decent Iteration
%--------------------------------
%
% Inputvariablen
%  - A    Zu untersuchende Matrix
%  - b    Lsg von Ax=b (Spaltenvektor)
%  - x0   Startvektor  (Spaltenvektor)
%  - imax Anzahl maximaler Iterationsschritte
%  - tol  Genauigkeit, bei deren Erreichen die Iteration abbricht
%         (Durch vergleich der relativen Residuen-Norm)
%
% Outputvariablen
%  - xk         Die letzte Iterierte
%
% Programmaufruf mit:
%
%   xk = steepestdecent(A,b,x0,imax,tol)
%
%   Die Parameter x0,imax und tol sind optional, falls nicht angegeben
%   werden die folgenden Standardwerte genutzt:
%
%     x0   = Nullvektor
%     imax = 1000
%     tol  = 1e-15
%
%
function  xk = steepestdecent(A,b,x0,imax,tol)

%% Falls nicht angegeben, Standardwerte festlegen
if (nargin < 2)
  error('steepestdecent:NotEnoughInputs', 'Not enough input arguments.');
end
if (nargin < 5)
  tol = 1e-15;
  if (nargin < 4)
    imax = 1000;
    if (nargin < 3)
      x0 = zeros(size(A,1),1);
    end
  end
end
xk = x0;

%% Initialisierung
iter = 0;
res = b-A*xk;
resnorm = norm(res);
bnorm = norm(b);

%% Iteration
while (iter < imax) && (resnorm/bnorm>tol)
    % neue Iterierte
    alpha = (res'*res)/(res'*A*res);
    xk = xk+alpha*res;
    % neues Residuum
    res = b-A*xk;
    resnorm = norm(res);
    iter = iter+1;
    if mod(iter,50)==0
       display(sprintf('iter=%d',iter));
    end
end